# util.py
# -*- coding: utf8 -*-
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# except for memory usage recipe.
# GPL - see COPYING for details

"""Utility module for use with pyobjects talk."""

import gc
import random
import subprocess
import time
import os
import sys

PYTHON_TRUNK_CHECKOUT = "/your/python/trunk"

#
# Old vs New Classes and Slots
#

class Old:
    """A classic or old-style class."""
    pass

class New(object):
    """A new-style class with a useful type."""
    pass

class FuBar(object):
    """Illustrates the difference between __str__ and
       __unicode__. __str__ has a slot (tp_str) while
       __unicode__ does not.
       """
    def __str__(self):
        return "fubar"
    def __unicode__(self):
        return unicode("fubar")

class OldFu:
    """Like FuBar, but showing it with classic classes."""
    def __str__(self):
        return "fubar"
    def __unicode__(self):
        return unicode("fubar")

#
# Grabbing the interned string dictionary
# and a simple Python keyword game
#

def find_big_dict(x):
    """Use gc to find the first dict with len greater than 1000
       that references x.
       """
    for obj in gc.get_referrers(x):
        if type(obj) is dict and len(obj) > 1000:
            return obj
    return None

def find_str_interns():
    """Find interned string dict by looking for a big dict that
       refers to "a".
       """
    return find_big_dict("a")

def keyword_quiz():
    """Looks up upper case keywords in the interned string dict
       and asks users to guess which part of Python they come
       from. Prints answers by grepping Python source.
       """
    interned = find_big_dict("a")
    keywords = [x for x in interned if x.isupper() and len(x) > 4]
    question = random.choice(keywords)

    pwd = os.getcwd()
    try:
        os.chdir(PYTHON_TRUNK_CHECKOUT)
        egrep = subprocess.Popen(["egrep", "-R", "-l", "--exclude-dir", ".svn",
                              "--include", "*.c", "--include", "*.h", "--include", "*.py",
                              '%s' % question,
                              "Modules", "Lib"], stdout=subprocess.PIPE)
        answer, err = egrep.communicate()
    finally:
        os.chdir(pwd)

    print ("Which module does the keyword " + question + " appear in?")
    for i in reversed(range(5)):
        sys.stdout.write(str(i) + " ... ")
        sys.stdout.flush()
        time.sleep(1)
    print ("")
    print ("-- Answers --")
    print (answer)

# Recipe for measuring process memory usage, taken from
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/286222.
# According to http://aspn.activestate.com/ASPN/Python/Cookbook/
# licensed under the Python license unless otherwise stated.
# I'm taking this to mean Python license 2.1 or newer which is
# GPL compatible.

_proc_status = '/proc/' + str(os.getpid()) + '/status'

_scale = {'kB': 1024.0, 'mB': 1024.0*1024.0,
          'KB': 1024.0, 'MB': 1024.0*1024.0}

def _VmB(VmKey):
    '''Private.
    '''
    global _proc_status, _scale
    # get pseudo file  /proc/<pid>/status
    try:
        t = open(_proc_status)
        v = t.read()
        t.close()
    except:
        return 0.0  # non-Linux?
    # get VmKey line e.g. 'VmRSS:  9999  kB\n ...'
    i = v.index(VmKey)
    v = v[i:].split(None, 3)  # whitespace
    if len(v) < 3:
        return 0.0  # invalid format?
    # convert Vm value to bytes
    return float(v[1]) * _scale[v[2]]

def memory(since=0.0):
    '''Return memory usage in bytes.
    '''
    return _VmB('VmSize:') - since


def resident(since=0.0):
    '''Return resident memory usage in bytes.
    '''
    return _VmB('VmRSS:') - since


def stacksize(since=0.0):
    '''Return stack size in bytes.
    '''
    return _VmB('VmStk:') - since

# END Recipe
